@@ -4,9 +4,9 @@ See PMP Docs for more information:
 https://github.com/publicmediaplatform/pmpdocs
 """
 
-from .core import exceptions
+from .core import exceptions as exc
 from .utils import json_utils
 from .pmp_client import Client
 from .collectiondoc.navigabledoc import NavigableDoc
 
-__version__ = '0.4.2'
+__version__ = '0.4.4'
