@@ -7,7 +7,6 @@ by the PMP API
 """
 from functools import wraps
 from itertools import dropwhile
-from pmp_api.core.exceptions import NoResult
 
 
 def qfind(json_dict, key):
@@ -15,6 +14,21 @@ def qfind(json_dict, key):
     Recursive method for finding nested dictionaries anywhere in a dictionary
     given a key which may or may not be in the dictionary.
 
+    This function actually returns the `parent` dictionary that contains a
+    particular key. Consider the following example::
+
+      >>> somedict = {'1': 1, '2': '2', 'a' : { '3' : 3}}
+      >>> next(qfind(somedict, '2'))  # key present: returns whole thing
+      {'2': '2', 'a': {'3': 3}, '1': 1}
+      >>> next(qfind(somedict, '3'))
+      {'3': 3}
+      >>> next(qfind(somedict, '4'))
+      Traceback (most recent call last):
+        File "<stdin>", line 1, in <module>
+      StopIteration
+      >>> list(qfind(somedict, '4'))
+      []
+
     Args:
        `json_dict` -- JSON dictionary.
        `key` -- Key we are searching for.
@@ -28,15 +42,15 @@ def qfind(json_dict, key):
             yield from qfind(json_dict[key], key)
         elif key in json_dict:
             yield json_dict
-        elif isinstance(json_dict, dict):
+        else:
             for k in json_dict:
                 yield from qfind(json_dict[k], key)
 
 
 def filter_dict(json_dict, key, val):
     """Returns a filter iterator from `json_dict` where results contain
-    `key` - `val` matches. Relies on qfind method to search out a particular
-    value inside the dictionary, so it works on nested dictionaries.
+    `key` - `val` matches. Relies on qfind method to find matching keys
+    inside the dictionary or list of dictionaries.
 
     Args:
        `json_dict` -- JSON dictionary.
@@ -46,11 +60,15 @@ def filter_dict(json_dict, key, val):
     qjson_dict = qfind(json_dict, key)
 
     def filterfunc(somedict):
-        if key in somedict and val in somedict[key]:
-            return True
-        elif key in somedict and somedict[key] == val:
-            return True
-        return False
+        try:
+            if somedict[key] == val or val in somedict[key]:
+                return True
+            else:
+                return False
+        except TypeError:
+            # TypeError: if the val passed in is an int and
+            # dict[key] is a string
+            return False
     return filter(filterfunc, qjson_dict)
 
 
@@ -58,30 +76,16 @@ def returnfirst(func):
     """Decorator for retrieving the first value of any function that returns
     multiple values or an iterator with more than one value.
 
+    This function returns None if noresults may be found.
+
     Args:
        `func` -- function that returns iterator
     """
-    @wraps
+    @wraps(func)
     def inner(*args, **kwargs):
         try:
             result, *_ = func(*args, **kwargs)
             return result
         except ValueError:
-            errmsg = "Result empty for provided arguments"
-            raise NoResult(errmsg.format(args, kwargs))
+            return None
     return inner
-
-
-# This is more trouble than it's worth probably.
-# Currently not used anywhere
-@returnfirst
-def get_dict(json_dict, key, val):
-    """Returns first dictionary that matches `key` - `val`
-    search. Unsafe if results are not guaranteed to appear.
-
-    Args:
-       `json_dict` -- JSON dictionary.
-       `key` -- Key we are searching for.
-       `val` -- Value that should explicitly match the key searched for.
-    """
-    return filter_dict(json_dict, key, val)
